using System.Collections.Generic;
using System.Linq;
using UnityEngine;

public static class ModResourcesLoader
{
    private static Dictionary<string, Sprite> sprites = new Dictionary<string, Sprite>();
    private static Dictionary<string, AudioClip> clips = new Dictionary<string, AudioClip>();
    private static Dictionary<string, Texture2D> textures = new Dictionary<string, Texture2D>();
	
	//Sprite
    public static void AddSprite(string name, string path, float scale = 1)
    {
        if (!sprites.ContainsKey(name)) sprites.Add(name, ModAPI.LoadSprite(path, scale));
    }
    
    public static Sprite GetSprite(string key)
    {
        return sprites.FirstOrDefault(x => x.Key == key).Value;
    }
	
	//AudioClip
    public static void AddSound(string name, string path)
    {
        if (!clips.ContainsKey(name)) clips.Add(name, ModAPI.LoadSound(path));
    }
    public static AudioClip GetSound(string key)
    {
        return clips.FirstOrDefault(x => x.Key == key).Value;
    }
    
	//Texture2D
    public static void AddTexture(string name, string path, bool pixelated = true)
    {
        if (!textures.ContainsKey(name)) textures.Add(name, ModAPI.LoadTexture(path, pixelated));
    }
    public static Texture2D GetTexture(string key)
    {
        return textures.FirstOrDefault(x => x.Key == key).Value;
    }
}