using System;
using System.Collections.Generic;
using System.Collections;
using UnityEngine;
using UnityEngine.Events;

namespace TaserMod
{
    public class TaserBehaviour : MonoBehaviour
    {
      private bool shoots = false;
      private bool stunning = false;
      private int step = 0;

      private TaserWireBehaviour Wire;
      private LimbBehaviour[] Limbs;
      private LimbBehaviour Limb;
      private AudioSource AS;
      private List<GameObject> Electrodes = new List<GameObject>();
	  
      public float distance = 4;
      public float shockDelay = .07f;
      public int shockCount = 30;
	  
	  private float heartFailureChance = .001f;

      Vector2 barrelPosition = new Vector2(.2f, .065f);
	  
	  private void PlaySound(string key)
      {
        AS.clip = ModResourcesLoader.GetSound(key);
        AS.Play();
      }

      private void Awake()
      {
        AS = gameObject.GetOrAddComponent<AudioSource>();

        PhysicalBehaviour phys = GetComponent<PhysicalBehaviour>();

        phys.ContextMenuOptions.Buttons.Add(new ContextMenuButton("distanceButton", "Distance", "Distance", () =>
        {
          DialogBox dialog = (DialogBox)null;
          dialog = DialogBoxManager.TextEntry("Enter distance.", "distance", new DialogButton("Apply", true, new UnityAction[1]
          {
            (UnityAction)(() => {
            float entered;
            if(float.TryParse(dialog.EnteredText, out entered))
			{
              if (entered > 0)
			  {
                distance = entered;
              }
            }
            })
          }),
          new DialogButton("Cancel", true, (UnityAction)(() => dialog.Close())));
        }));

        phys.ContextMenuOptions.Buttons.Add(new ContextMenuButton("delayButton", "Shock delay", "Shock delay", () =>
        {
          DialogBox dialog = (DialogBox)null;
          dialog = DialogBoxManager.TextEntry("Enter shock delay.", "delay", new DialogButton("Apply", true, new UnityAction[1]
          {
            (UnityAction)(() =>
			{
				float entered;
				if(float.TryParse(dialog.EnteredText, out entered))
				{
				  if (entered > 0)
				  {
					shockDelay = entered;
				  }
				}
				})
          }),
          new DialogButton("Cancel", true, (UnityAction)(() => dialog.Close())));
        }));

        phys.ContextMenuOptions.Buttons.Add(new ContextMenuButton("shockCountButton", "Shock count", "Shock count", () =>
        {
          DialogBox dialog = (DialogBox)null;
          dialog = DialogBoxManager.TextEntry("Enter shock count.", "count", new DialogButton("Apply", true, new UnityAction[1]
          {
            (UnityAction)(() =>
			{
				int entered;
				if(int.TryParse(dialog.EnteredText, out entered))
				{
				  if (entered > 0)
				  {
					shockCount = entered;
				  }
				}
            })
          }),
          new DialogButton("Cancel", true, (UnityAction)(() => dialog.Close())));
        }));
      }
	  
	  private void Start() => TaserReset();

      private void CreateElectrode(Vector3 pos, Transform parent)
      {
        GameObject electrode = new GameObject();
        electrode.transform.position += new Vector3(pos.x + UnityEngine.Random.Range(-.05f, .05f), pos.y + UnityEngine.Random.Range(-.05f, .05f), pos.z);
        electrode.AddComponent<SpriteRenderer>().sprite = ModResourcesLoader.GetSprite("electrode");
        electrode.transform.eulerAngles = new Vector3(0, 0, UnityEngine.Random.Range(-45, 45));
        electrode.transform.SetParent(parent);
        electrode.transform.localScale = new Vector3(.4f, .4f);
        Electrodes.Add(electrode);
      }

      private void TaserReset()
      {
        if (GetComponent<TaserWireBehaviour>())
        {
          GameObject.Destroy(GetComponent<TaserWireBehaviour>());
        }
        if (GetComponent<DistanceJoint2D>())
        {
          GameObject.Destroy(GetComponent<DistanceJoint2D>());
        }
        shoots = false;
        stunning = false;
        step = 0;
        Wire = null;
        Limbs = null;
        if (Electrodes.Count > 0)
        {
          GameObject.Destroy(Electrodes[0]);
          GameObject.Destroy(Electrodes[1]);
          Electrodes.Clear();
        }
      }
	  
      private void Use()
      {
        if (stunning) return;
        step++;
        switch (step)
        {
          case 1:
            Shoot();
            break;
          case 2:
            if (Limb != null)
            {
              StartCoroutine(Stun());
            }
            else
            {
              TaserReset();
            }
            break;
          case 3:
            step = 0;
            break;
        }
      }

      private void Shoot()
      {
        RaycastHit2D ray = Physics2D.Raycast(transform.position, transform.right, distance * transform.localScale.x, LayerMask.GetMask("Objects"));
        if (ray.collider == null)
        {
          step = 0;
          return;
        }
        if (ray.transform.GetComponent<LimbBehaviour>())
        {
          Limbs = ray.transform.GetComponent<LimbBehaviour>().Person.Limbs;
          Limb = ray.transform.GetComponent<LimbBehaviour>();
          PlaySound("shoot");
          shoots = true;
          Attach(ray);
        }
        else
        {
          step = 0;
        }
      }

      private void Attach(RaycastHit2D hit)
      {
        DistanceJoint2D joint = gameObject.AddComponent<DistanceJoint2D>();
        joint.maxDistanceOnly = true;
        joint.enableCollision = false;
        joint.autoConfigureDistance = false;
        joint.distance = distance;
        joint.breakForce = Mathf.Infinity;
        joint.breakTorque = Mathf.Infinity;
        joint.anchor = barrelPosition;
        joint.connectedBody = hit.rigidbody;
        joint.connectedAnchor = hit.transform.InverseTransformPoint(hit.point);
        TaserWireBehaviour wireBehaviour = gameObject.AddComponent<TaserWireBehaviour>();
        wireBehaviour.WireColor = Color.white;
        wireBehaviour.WireMaterial = Resources.Load<Material>("Materials/Wire");
        wireBehaviour.WireWidth = .002f;
        wireBehaviour.typedJoint = joint;
        Wire = wireBehaviour;
        int n = 0;
        while (n < 2)
        {
            CreateElectrode(hit.point, hit.transform);
            n++;
        }
      }

      private IEnumerator Stun()
      {
        stunning = true;
        for (int i = 0; i < shockCount; i++)
        {
          if (shoots)
          {
            Shock();
            StartCoroutine(WireAnim());
            yield return new WaitForSeconds(shockDelay);
          }
          else
          {
            StopCoroutine(Stun());
          }
        }
        TaserReset();
      }

      private IEnumerator WireAnim()
      {
        Wire.stun = true;
        yield return new WaitForSeconds(shockDelay / 3);
        Wire.stun = false;
      }

      private void Shock()
      {
        float rand = UnityEngine.Random.value;
        if (Limb != null)
        {
          if (Wire)
          {
            for (int i = 0; i < Limbs.Length; i++)
            {
              if (rand <= heartFailureChance)
              {
                if (!Limb.IsAndroid)
                {
                  if (Limb.name == "UpperBody" || Limb.name == "MiddleBody")
                  {
                    Limbs[0].Health = 0;
                  }
                }
              }
              if (Limbs[i] != null)
              {
                Limbs[i].PhysicalBehaviour.charge = .25f;
                Limbs[i].PhysicalBehaviour.rigidbody.AddForce(Vector3.down * .3f);
                if (Limbs[i].GripBehaviour)
                {
                  Limbs[i].GripBehaviour.DropObject();
                }
                if (i > 0)
                {
                  Limbs[i].GetComponent<Rigidbody2D>().AddForce(UnityEngine.Random.insideUnitCircle.normalized * 10);
                }
              }
              Limb.Person.ShockLevel += UnityEngine.Random.Range(.00035f, .0035f);
              Limb.Person.Consciousness -= UnityEngine.Random.Range(.0004f, .0010f);
            }
            PlaySound("stun");
          }
        }
        else
        {
          TaserReset();
        }
      }
    }

    public class TaserWireBehaviour : DistanceJointWireBehaviour
    {
      public bool stun = false;
	  
      protected override void Tick()
      {
        if (stun)
        {
			lineRenderer.startColor = Color.LerpUnclamped(this.WireColor, Color.cyan, UnityEngine.Random.value * 10);
			lineRenderer.endColor = Color.LerpUnclamped(this.WireColor, Color.cyan, UnityEngine.Random.value * 10);
        }
      }
    }
  }
